# -*- coding: utf-8 -*-
"""Metadados do projeto."""

__version__ = '1.0.3'
__author__ = 'Matheus Felipe'
__email__ = 'matheusfelipeog@protonmail.com'
__author_github__ = 'https://github.com/matheusfelipeog'
__project_github__ = 'https://github.com/matheusfelipeog/fordev'
